package models

import (
	"eta_gn/eta_task/global"
	"github.com/rdlucklib/rdluck_tools/paging"
	"time"
)

type Classify struct {
	Id             int       `gorm:"column:id;primaryKey"` // `orm:"column(id);pk"`
	ClassifyName   string    `description:"分类名称"`
	Sort           int       `json:"-"`
	ParentId       int       `description:"父级分类id"`
	CreateTime     time.Time `description:"创建时间"`
	ModifyTime     time.Time `description:"修改时间"`
	Abstract       string    `description:"栏目简介"`
	Descript       string    `description:"分享描述"`
	ReportAuthor   string    `description:"栏目作者"`
	AuthorDescript string    `description:"作者简介"`
	ColumnImgUrl   string    `description:"栏目配图"`
	HeadImgUrl     string    `description:"头部banner"`
	AvatarImgUrl   string    `description:"头像"`
	ReportImgUrl   string    `description:"报告配图"`
	HomeImgUrl     string    `description:"首页配图"`
	ClassifyLabel  string    `description:"分类标签"`
	IsMassSend     int       `description:"1:群发,0:非群发"`
}

type ClassifyAddReq struct {
	ClassifyName   string `description:"分类名称"`
	ParentId       int    `description:"父级分类id，没有父级分类传0"`
	Abstract       string `description:"栏目简介"`
	Descript       string `description:"分享描述"`
	ReportAuthor   string `description:"栏目作者"`
	AuthorDescript string `description:"作者简介"`
	ColumnImgUrl   string `description:"栏目配图"`
	ReportImgUrl   string `description:"报告配图"`
	HeadImgUrl     string `description:"头部banner"`
	AvatarImgUrl   string `description:"头像"`
	HomeImgUrl     string `description:"首页配图"`
	ClassifyLabel  string `description:"分类标签"`
}


func GetClassifyById(classifyId int) (item *Classify, err error) {

	sql := `SELECT * FROM classify WHERE id=? `
	err = global.DmSQL["data"].Raw(sql, classifyId).Find(&item).Error
	return
}


type ClassifyList struct {
	Id            int         `gorm:"column:id;primaryKey"` // `orm:"column(id);pk"`
	ClassifyName  string      `description:"分类名称"`
	Sort          int         `json:"-"`
	ParentId      int         `description:"父级分类id"`
	CreateTime    time.Time   `description:"创建时间"`
	ModifyTime    time.Time   `description:"修改时间"`
	Abstract      string      `description:"简介"`
	Descript      string      `description:"描述"`
	ClassifyLabel string      `description:"分类标签"`
	Child         []*Classify `gorm:"-"`
}

type ClassifyListResp struct {
	List   []*ClassifyList    `gorm:"-"`
	Paging *paging.PagingItem `description:"分页数据"`
}


type CheckDeleteClassifyReq struct {
	ClassifyId int `description:"分类ID"`
}

type CheckDeleteClassifyResp struct {
	Code int    `description:"编码：0:检测成功，可进行删除,1：分类不存在，2：该分类有关联报告，不允许删除,3:二级分类有关联报告，不允许删除,4:该分类下有关联分类，是否确认全部删除"`
	Msg  string `description:"描述信息"`
}

type DeleteClassifyReq struct {
	ClassifyId int `description:"分类ID"`
}

type EditClassifyReq struct {
	ClassifyId     int    `description:"分类ID"`
	ClassifyName   string `description:"分类名称"`
	ParentId       int    `description:"父级分类id"`
	Abstract       string `description:"栏目简介"`
	Descript       string `description:"分享描述"`
	ReportAuthor   string `description:"栏目作者"`
	AuthorDescript string `description:"作者简介"`
	ColumnImgUrl   string `description:"栏目配图"`
	HeadImgUrl     string `description:"头部banner"`
	AvatarImgUrl   string `description:"头像"`
	ReportImgUrl   string `description:"报告配图"`
	HomeImgUrl     string `description:"首页配图"`
	ClassifyLabel  string `description:"分类标签"`
}

type FindByIdClassifyReq struct {
	ClassifyId int `description:"分类ID"`
}

