package edb_refresh

import (
	"errors"
	"eta_gn/eta_task/global"
	"eta_gn/eta_task/utils"
	"time"
)

type EdbRefreshDefaultConfig struct {
	Id                  int       `gorm:"column:id;primaryKey"` //   `orm:"column(id);pk"`
	Source              int       `description:"来源"`
	SubSource           int       `description:"来源名称"`
	Frequency           string    `description:"频度"`
	RefreshFrequency    string    `description:"刷新频率"`
	RefreshFrequencyDay int       `description:"具体刷新的日期"`
	RefreshTime         string    `description:"刷新时间"`
	RefreshAllData      int       `description:"是否刷新所有数据，0：否，1：刷新所有数据"`
	RefreshDataNum      int       `description:"刷新单元格数"`
	ModifyTime          time.Time `description:"最晚一次的更新时间"`
	CreateTime          time.Time `description:"添加时间"`
}

func (m *EdbRefreshDefaultConfig) Add() (err error) {
	if m.Id > 0 {
		err = errors.New("该配置已存在")
		return
	}
	err = global.DmSQL["data"].Create(m).Error
	return
}

func (m *EdbRefreshDefaultConfig) Update(cols []string) (err error) {

	err = global.DmSQL["data"].Select(cols).Updates(m).Error
	return
}

func (m *EdbRefreshDefaultConfig) Delete() (err error) {
	err = global.DmSQL["data"].Exec("DELETE FROM edb_refresh_default_config WHERE id = ?", m.Id).Error
	return
}

func GetListBySourceAndFrequency(source, subSource int, frequency string) (list []*EdbRefreshDefaultConfig, err error) {
	sql := `SELECT * FROM edb_refresh_default_config
         WHERE source = ? AND sub_source = ? AND frequency = ? ORDER BY id ASC `
	err = global.DmSQL["data"].Raw(sql, source, subSource, frequency).Find(&list).Error

	return
}

func GetListByCondition(condition string, pars []interface{}) (list []*EdbRefreshDefaultConfig, err error) {
	sql := `SELECT * FROM edb_refresh_default_config
         WHERE 1 = 1 `

	if condition != "" {
		sql += condition
	}
	sql += ` ORDER BY id ASC `
	err = global.DmSQL["data"].Raw(sql, pars...).Find(&list).Error

	return
}

func GetDefaultRefreshEdbInfoListBySourceAndSubSource(source, subSource int, frequencyList []string) (list []*EdbInfoListAndRefreshConfig, err error) {
	num := len(frequencyList)
	if num <= 0 {
		return
	}

	sql := `SELECT a.* FROM edb_info a 
LEFT JOIN edb_refresh_mapping b ON a.edb_info_id = b.edb_info_id
WHERE a.source = ? AND a.sub_source = ? AND a.frequency IN (` + utils.GetOrmInReplace(num) + `)  AND b.edb_info_id is null`
	err = global.DmSQL["data"].Raw(sql, source, subSource, frequencyList).Find(&list).Error
	return
}

func GetDistinctDefaultRefreshSourceIds() (sourceIds []int, err error) {
	sql := `SELECT DISTINCT source FROM edb_refresh_default_config`
	err = global.DmSQL["data"].Raw(sql).Scan(&sourceIds).Error
	return
}
