package future_good

import (
	"eta_gn/eta_task/global"
	"time"
)

type FutureGoodEdbInfo struct {
	FutureGoodEdbInfoId int       `gorm:"column:future_good_edb_info_id;primaryKey"` // `orm:"column(future_good_edb_info_id);pk"`
	FutureGoodEdbCode   string    `description:"期货指标code"`
	FutureGoodEdbName   string    `description:"期货指标名称"`
	FutureGoodEdbNameEn string    `description:"期货指标英文名称"`
	ParentId            int       `description:"上级期货id"`
	Exchange            string    `description:"所属交易所"`
	Month               int       `description:"所属月份"`
	StartDate           string    `description:"起始日期"`
	EndDate             time.Time `description:"终止日期"`
	MinValue            float64   `description:"最小值"`
	MaxValue            float64   `description:"最大值"`
	LatestValue         float64   `description:"数据最新的值"`
	LatestDate          time.Time `description:"数据最新的日期"`
	ServerUrl           string    `description:"服务器地址"`
	CreateTime          time.Time
	ModifyTime          time.Time
}

func GetFutureGoodEdbInfo(edbInfoId int) (item *FutureGoodEdbInfo, err error) {
	sql := `SELECT * FROM future_good_edb_info WHERE future_good_edb_info_id = ? `
	sql += ` ORDER BY future_good_edb_info_id DESC `
	err = global.DEFAULT_DmSQL.Raw(sql, edbInfoId).First(&item).Error
	return
}

func GetFutureGoodEdbInfoList(condition string, pars []interface{}) (list []*FutureGoodEdbInfo, err error) {
	sql := `SELECT * FROM future_good_edb_info WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	sql += ` ORDER BY future_good_edb_info_id DESC `
	err = global.DEFAULT_DmSQL.Raw(sql, pars...).Find(&list).Error
	return
}

func GetAllFutureGoodEdbInfoList() (list []*FutureGoodEdbInfo, err error) {
	sql := `SELECT * FROM future_good_edb_info  ORDER BY future_good_edb_info_id DESC `
	err = global.DEFAULT_DmSQL.Raw(sql).Find(&list).Error
	return
}

func GetFutureGoodEdbInfoListByParentId(parentId int) (list []*FutureGoodEdbInfo, err error) {
	sql := `SELECT * FROM future_good_edb_info WHERE parent_id = ?  ORDER BY future_good_edb_info_id ASC `
	err = global.DEFAULT_DmSQL.Raw(sql, parentId).Find(&list).Error
	return

}

func AddFutureGoodEdbInfo(item *FutureGoodEdbInfo) (err error) {

	err = global.DEFAULT_DmSQL.Create(item).Error
	return
}

func (FutureGoodEdbInfo *FutureGoodEdbInfo) Update(cols []string) (err error) {
	err = global.DEFAULT_DmSQL.Select(cols).Updates(FutureGoodEdbInfo).Error
	return
}

type FutureGoodEdbInfoGroupListResp struct {
	FutureGoodEdbInfoId   int
	FutureGoodEdbInfoName string
	Child                 []FutureGoodEdbInfoGroupListResp
}

func GetFutureGoodEdbInfoGroupList(condition string, pars []interface{}) (list []*FutureGoodEdbInfo, err error) {
	sql := `SELECT * FROM future_good_edb_info WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	sql += ` ORDER BY future_good_edb_info_id DESC `

	err = global.DEFAULT_DmSQL.Raw(sql, pars...).Find(&list).Error
	return
}

type BarChartInfoReq struct {
	EdbInfoIdList []BarChartInfoEdbItemReq `description:"指标信息"`
	DateList      []BarChartInfoDateReq    `description:"日期配置"`
	Sort          BarChartInfoSortReq      `description:"排序"`
}

type BarChartInfoEdbItemReq struct {
	EdbInfoId int    `description:"指标ID"`
	Name      string `description:"别名"`
	Source    int    `description:"1：ETA图库；2：商品价格"`
}

type BarChartInfoDateReq struct {
	Type  int    `description:"配置类型"`
	Date  string `description:"固定日期"`
	Value int    `description:"N天的值"`
	Color string `description:"颜色"`
	Name  string `description:"别名"`
}

type BarChartInfoSortReq struct {
	Sort      int `description:"排序类型，0：默认，1：升序，2：降序"`
	DateIndex int `description:"日期数据的下标，从0开始"`
}
