package report

import (
	"eta_gn/eta_task/global"
	"eta_gn/eta_task/utils"
	"time"
)

type ReportChapterPermissionMapping struct {
	ReportChapterPermissionMappingId int `orm:"column(report_chapter_permission_mapping_id)"`
	ReportChapterId                  int `description:"报告章节的id"` // 报告章节的id
	ChartPermissionId                int `description:"权限id"`    // 权限id
	CreateTime                       time.Time
}

func (m ReportChapterPermissionMapping) MultiAddReportChapterPermissionMappingPermission(reportChapterId int, list []*ReportChapterPermissionMapping) (err error) {
	to := global.DmSQL["rddp"].Begin()

	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()
	sql := "DELETE from report_chapter_permission_mapping where report_chapter_id=?"
	err = to.Exec(sql, reportChapterId).Error
	if err != nil {
		return
	}

	if len(list) > 0 {

		tmpErr := to.CreateInBatches(list, utils.MultiAddNum).Error
		if tmpErr != nil {
			err = tmpErr
			return
		}
	}

	return
}

func (m ReportChapterPermissionMapping) GetPermissionListById(reportChapterId int) (list []*ReportChapterPermissionMapping, err error) {

	sql := `SELECT * FROM report_chapter_permission_mapping WHERE report_chapter_id=? `
	err = global.DmSQL["rddp"].Raw(sql, reportChapterId).Find(&list).Error
	return
}

type ReportChapterPermissionItem struct {
	ReportChapterPermissionMappingId int    `orm:"column(report_chapter_permission_mapping_id)"`
	ReportChapterId                  int    `description:"报告章节的id"`
	ChartPermissionId                int    `description:"权限id"`
	ChartPermissionName              string `description:"品种名称"`
	CreateTime                       time.Time
}

func (m ReportChapterPermissionMapping) GetPermissionItemListById(reportChapterId int) (list []*ReportChapterPermissionItem, err error) {

	sql := `SELECT a.*,b.chart_permission_name FROM report_chapter_permission_mapping AS a 
	    JOIN chart_permission AS b on a.chart_permission_id=b.chart_permission_id WHERE report_chapter_id=? `
	err = global.DmSQL["data"].Raw(sql, reportChapterId).Find(&list).Error
	return
}

func (m ReportChapterPermissionMapping) GetPermissionListByIdList(reportChapterIdList []int) (list []*ReportChapterPermissionMapping, err error) {
	num := len(reportChapterIdList)
	if num <= 0 {
		return
	}


	sql := `SELECT * FROM report_chapter_permission_mapping WHERE report_chapter_id in (` + utils.GetOrmInReplace(num) + `) `
	err = global.DmSQL["data"].Raw(sql, reportChapterIdList).Find(&list).Error
	return
}

func (m ReportChapterPermissionMapping) MultiAdd(list []*ReportChapterPermissionMapping) (err error) {
	to := global.DmSQL["rddp"].Begin()

	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()
	if len(list) > 0 {
		err = to.CreateInBatches(list, utils.MultiAddNum).Error
		if err != nil {
			return
		}
	}

	return
}
